import 'dart:developer' as dev;

import 'package:flutter/material.dart';
import 'package:flutter_gadgets_2/flutter_gadgets_2.dart';

import '../controller/my_home_view_controller.dart';
import '../model/alien_counter.dart';

class MyHomeView extends StatelessWidget {
  const MyHomeView({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: const Text('Alien Counter'),
        centerTitle: false,
      ),
      body: ViewGadget<MyHomeViewController>(
        controller: MyHomeViewController(),
        builder: (_, controller) => Column(
          crossAxisAlignment: CrossAxisAlignment.center,
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            SubscriberGadget<AlienCounter>(
              propertiesSelector: (counter) => [counter.aliens],
              builder: (context, counter) {
                dev.log('Building for AlienCounter.aliens');
                return Text(counter.alienWarning);
              },
            ),
            const SizedBox(height: 16),
            SubscriberGadget<AlienCounter>(
              propertiesSelector: (counter) => [counter.humans],
              builder: (context, counter) {
                dev.log('Building for AlienCounter.humans');
                return Text(counter.humanWarning);
              },
            ),
            const SizedBox(height: 16),
            SubscriberGadget<AlienCounter>(
              builder: (context, counter) {
                dev.log('Building for AlienCounter');
                return Text(counter.livingCreature);
              },
            ),
            const SizedBox(height: 16),
            Row(
              mainAxisAlignment: MainAxisAlignment.spaceEvenly,
              children: <Widget>[
                FloatingActionButton(
                  child: const Text('😱'),
                  onPressed: controller.onAddAlien,
                ),
                FloatingActionButton(
                  child: const Text('👽'),
                  onPressed: controller.onAddHuman,
                ),
              ],
            ),
          ],
        ),
      ),
    );
  }
}
