import 'package:flutter/material.dart';
import 'package:flutter_gadgets_2/flutter_gadgets_2.dart';

import '../controller/my_game_view_controller.dart';
import '../model/game.dart';

class MyGameViewModel {
  late TextEditingController controller;
  String? errorMessage;
  bool canPlay = true;
}

class MyGameView extends StatelessWidget {
  const MyGameView({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return ViewGadget<MyGameViewController>(
      controller: MyGameViewController(),
      builder: (_, control) => ViewModelGadget<MyGameViewModel>(
        viewModel: MyGameViewModel(),
        init: (viewModel) => viewModel.controller = TextEditingController(),
        dispose: (viewModel) => viewModel.controller.dispose(),
        builder: (_, viewModel) {
          return Scaffold(
            appBar: AppBar(
              title: const Text('Game'),
              centerTitle: false,
              actions: [
                IconButton(
                  icon: const Icon(Icons.refresh),
                  onPressed: control.onResetPressed,
                )
              ],
            ),
            body: Padding(
              padding: const EdgeInsets.all(8.0),
              child: Column(
                children: [
                  SubscriberGadget<Game>(
                    builder: (_, game) => Text(game.hint),
                  ),
                  const SizedBox(height: 16),
                  Row(
                    mainAxisSize: MainAxisSize.max,
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: [
                      const Text('Attempts:'),
                      SubscriberGadget<Game>(
                        builder: (_, game) => Text(
                          game.attempts.toString(),
                        ),
                      ),
                    ],
                  ),
                  const SizedBox(height: 32),
                  SubscriberGadget<MyGameViewModel>(
                    builder: (_, viewModel) => TextField(
                      controller: viewModel.controller,
                      enabled: viewModel.canPlay,
                      keyboardType: TextInputType.number,
                      decoration: InputDecoration(
                        hintText: 'Insert a number',
                        errorText: viewModel.errorMessage,
                      ),
                    ),
                  )
                ],
              ),
            ),
            floatingActionButton: SubscriberGadget<MyGameViewModel>(
              propertiesSelector: (viewModel) => [viewModel.canPlay],
              builder: (_, viewModel) => FloatingActionButton(
                onPressed: viewModel.canPlay ? control.onNextTurnPressed : null,
                child: const Icon(Icons.arrow_right_alt),
                backgroundColor: !viewModel.canPlay ? Colors.grey : null,
              ),
            ),
          );
        },
      ),
    );
  }
}
