import 'dart:math' as math;

import 'package:flutter/material.dart';
import 'package:flutter_gadgets_2/flutter_gadgets_2.dart';

import '../const.dart';
import '../controller/my_game_view_controller.dart';
import '../model/match.dart';
import '../model/utils.dart';

class MyGameView extends StatelessWidget {
  const MyGameView({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return ViewGadget<MyGameViewController>(
      controller: MyGameViewController(),
      builder: (_, control) => Scaffold(
        appBar: AppBar(
          title: const Text('Rock Paper Scissor'),
          centerTitle: false,
          actions: [
            IconButton(
              icon: const Icon(Icons.refresh),
              onPressed: control.onRefreshPressed,
            )
          ],
        ),
        body: Padding(
          padding: const EdgeInsets.all(8.0),
          child: SubscriberGadget<Match>(
            builder: (_, match) => Column(
              crossAxisAlignment: CrossAxisAlignment.center,
              children: [
                const Text(
                  'Play a match!',
                  style: TextStyle(
                    fontWeight: FontWeight.w700,
                  ),
                ),
                Text(match.matchStateDescription),
                Text(match.lastTurnStateDescription),
                const SizedBox(height: 16),
                RowText('Player wins:', match.playerWins.toString()),
                RowText('Ties:', match.ties.toString()),
                RowText('Cpu wins:', match.cpuWins.toString()),
                const SizedBox(height: 16),
                Row(
                  mainAxisAlignment: MainAxisAlignment.spaceAround,
                  children: [
                    TextButton(
                      'ROCK',
                      () => control.onButtonPressed(HandSign.rock),
                      match.matchState == MatchState.playing,
                    ),
                    TextButton(
                      'PAPER',
                      () => control.onButtonPressed(HandSign.paper),
                      match.matchState == MatchState.playing,
                    ),
                    TextButton(
                      'SCISSOR',
                      () => control.onButtonPressed(HandSign.scissor),
                      match.matchState == MatchState.playing,
                    ),
                  ],
                ),
                const SizedBox(height: 16),
                const HandSignRow()
              ],
            ),
          ),
        ),
      ),
    );
  }
}

class RowText extends StatelessWidget {
  const RowText(this.firstText, this.secondText, {Key? key}) : super(key: key);

  final String firstText;
  final String secondText;

  @override
  Widget build(BuildContext context) {
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: [
        Text(firstText),
        Text(
          secondText,
          style: const TextStyle(fontWeight: FontWeight.bold),
        ),
      ],
    );
  }
}

class TextButton extends StatelessWidget {
  const TextButton(this.text, this.onPressed, this.enabled, {Key? key})
      : super(key: key);

  final String text;
  final VoidCallback onPressed;
  final bool enabled;

  @override
  Widget build(BuildContext context) {
    return MaterialButton(
      color: Theme.of(context).primaryColor,
      disabledColor: Colors.grey,
      onPressed: enabled ? onPressed : null,
      child: Text(
        text,
        style: const TextStyle(color: Colors.white),
      ),
    );
  }
}

class HandSignRow extends StatelessWidget {
  const HandSignRow({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: [
        SubscriberGadget<Match>(
          builder: (_, match) {
            return _image(match.lastTurn?.playerChoice);
          },
        ),
        Transform(
          alignment: Alignment.center,
          transform: Matrix4.rotationY(math.pi),
          child: SubscriberGadget<Match>(
            builder: (_, match) {
              return _image(match.lastTurn?.cpuChoice);
            },
          ),
        ),
      ],
    );
  }

  Widget _image(HandSign? sign) {
    if (sign == null) return Container();
    if (sign == HandSign.paper) return Image.asset(imagePathPaper);
    if (sign == HandSign.rock) return Image.asset(imagePathRock);
    return Image.asset(imagePathScissor);
  }
}
