import 'package:flutter/widgets.dart';
import 'package:provider/provider.dart';

/// A function that creates an object of type [T].
typedef Create<T> = T Function(BuildContext context);

/// A function that disposes an object of type [T].
typedef Dispose<T> = void Function(BuildContext context, T value);

/// A simple utility class that encapsulates a [Create] and a [Dispose] function
/// and generates a [Provider] in order to make the object created
/// with the [Create] function available in the widget tree.
///
/// A list of services can be specified when creating a new [AppGadget]
/// widget:
///
/// ```dart
/// AppGadget(
///  services: [
///     Service(create: (_) => MyCustomObject()),
///     Service(create: (_) => MyOtherCustomObject()),
///  ]
///  child: MyChildWidget()
/// );
/// ```
class Service<T> {
  Service({required this.create, this.dispose});

  Create<T> create;
  Dispose<T>? dispose;

  Provider<T> get provider => Provider<T>(create: create, dispose: dispose);
}
