import 'package:flutter/material.dart';
import 'package:flutter_gadgets_2/flutter_gadgets_2.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:provider/provider.dart';

import '../utils.dart';

void main() {
  testWidgets('AppGadget tree construction', (tester) async {
    await tester
        .pumpWidget(AppGadget(child: Builder(builder: (_) => Container())));

    expect(
        Provider.of<ObservableModel>(
          tester.element(find.byType(Container)),
          listen: false,
        ),
        isNotNull);

    await tester.pumpWidget(Container());
  });

  testWidgets('AppGadget tree construction with service', (tester) async {
    await tester.pumpWidget(AppGadget(
      services: [Service<MockService>(create: (_) => MockService())],
      child: Builder(builder: (_) => Container()),
    ));

    expect(
        Provider.of<MockService>(
          tester.element(find.byType(Container)),
          listen: false,
        ),
        isNotNull);
    expect(
        Provider.of<ObservableModel>(
          tester.element(find.byType(Container)),
          listen: false,
        ).getValue<MockValue>(),
        isNull);

    await tester.pumpWidget(Container());
  });

  testWidgets('AppGadget tree construction with bean', (tester) async {
    await tester.pumpWidget(AppGadget(
      init: (model) => model.putValue<MockValue>(value: MockValue()),
      child: Builder(builder: (_) => Container()),
    ));

    expect(
        Provider.of<ObservableModel>(
          tester.element(find.byType(Container)),
          listen: false,
        ).getValue<MockValue>(),
        isNotNull);

    await tester.pumpWidget(Container());
  });
}
