import 'package:flutter/material.dart';
import 'package:flutter_gadgets_2/flutter_gadgets_2.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:provider/provider.dart';

import '../utils.dart';

class MockController extends ControllerGadget {
  void onTap() {
    final value = model.getValue<MockValue>()!;
    value.increment();
  }
}

void main() {
  testWidgets('SubscriberWidget value', (tester) async {
    await tester.pumpWidget(AppGadget(
      init: (model) => model.putValue<MockValue>(value: MockValue()),
      child: SubscriberGadget<MockValue>(
        builder: (_, value) {
          return Text(
            '${value.state}',
            textDirection: TextDirection.ltr,
          );
        },
      ),
    ));
    expect(find.text('0'), findsOneWidget);

    await tester.pumpWidget(Container());
  });

  testWidgets('SubscriberWidget value changed with notify', (tester) async {
    final key = UniqueKey();
    await tester.pumpWidget(AppGadget(
      init: (model) => model.putValue<MockValue>(value: MockValue()),
      child: ViewGadget<MockController>(
        controller: MockController(),
        builder: (_, controller) => SubscriberGadget<MockValue>(
          builder: (_, value) {
            return GestureDetector(
              onTap: controller.onTap,
              child: Text(
                '${value.state}',
                key: key,
                textDirection: TextDirection.ltr,
              ),
            );
          },
        ),
      ),
    ));
    await tester.tap(find.byKey(key));
    expect(
        Provider.of<ObservableModel>(
          tester.element(find.byType(Text)),
          listen: false,
        ).getValue<MockValue>()!.state,
        1);
    await tester.pump();
    expect(find.text('1'), findsOneWidget);

    await tester.pumpWidget(Container());
  });

  testWidgets('SubscriberWidget value changed without notify', (tester) async {
    final key = UniqueKey();
    await tester.pumpWidget(AppGadget(
      init: (model) => model.putValue<MockValue>(value: MockValue()),
      child: ViewGadget<MockController>(
        controller: MockController(),
        builder: (_, controller) => SubscriberGadget<MockValue>(
          builder: (_, bean) {
            return GestureDetector(
              onTap: controller.onTap,
              child: Text(
                '${bean.state}',
                key: key,
                textDirection: TextDirection.ltr,
              ),
            );
          },
        ),
      ),
    ));
    final model = Provider.of<ObservableModel>(
      tester.element(find.byType(Text)),
      listen: false,
    );
    final bean = model.getValue<MockValue>()!;
    bean.increment();
    expect(model.getValue<MockValue>()!.state, 1);
    await tester.tap(find.byKey(key));
    await tester.pump();
    expect(find.text('1'), findsNothing);

    await tester.pumpWidget(Container());
  });
}
